#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from datetime import datetime
from typing import Union

from .common import CatalogModel


class Address(CatalogModel):
    company: str
    country_code_alpha2: str
    country_code_alpha3: str
    country_code_numeric: str
    country_name: str
    created_at: datetime
    customer_id: str
    extended_address: str
    first_name: str
    id: str
    last_name: str
    locality: str
    postal_code: str
    region: str
    street_address: str
    updated_at: datetime


class CreditCard(CatalogModel):
    """
    https://developer.paypal.com/braintree/docs/reference/response/credit-card
    """

    billing_address: Address
    bin: str
    card_type: str
    cardholder_name: str
    commercial: str
    country_of_issuance: str
    created_at: datetime
    customer_id: str
    customer_location: str
    debit: str
    default: bool
    durbin_regulated: str
    expiration_date: str
    expiration_month: str
    expiration_year: str
    expired: bool
    healthcare: str
    image_url: str
    issuing_bank: str
    last_4: str
    masked_number: str
    payroll: str
    prepaid: str
    product_id: str
    token: str
    unique_number_identifier: str
    updated_at: datetime


class ApplePayCard(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/apple-pay-card
    """

    source_description: str
    payment_instrument_name: str


class SamsungPayCard(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/samsung-pay-card
    """


class MasterpassCard(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/masterpass-card
    """


class AndroidPayCard(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/android-pay-card
    """

    google_transaction_id: str
    source_card_type: str
    source_description: str
    is_network_tokenized: bool
    source_card_last_4: str
    source_card_type: str
    virtual_card_last_4: str
    virtual_card_type: str


class VisaCheckoutCard(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/visa-checkout-card
    """


class VenmoAccount(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/venmo-account
    """

    source_description: str
    username: str
    venmo_user_id: str


class PayPalAccount(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/paypal-account
    """

    billing_agreement_id: str
    email: str
    payer_id: str
    revoked_at: datetime


class USBankAccount(CreditCard):
    """
    https://developer.paypal.com/braintree/docs/reference/response/us-bank-account
    """

    account_holder_name: str
    account_type: str
    ach_mandate: str
    bank_name: str
    business_name: str
    last_name: str
    owner_id: str
    ownership_type: str
    plaid_verified_at: datetime
    routing_number: str
    verifiable: bool
    verified: bool


PaymentMethod = Union[
    CreditCard, AndroidPayCard, ApplePayCard, SamsungPayCard, USBankAccount, PayPalAccount, VenmoAccount, VisaCheckoutCard
]



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from datetime import date, datetime
from decimal import Decimal
from typing import List

from .common import AddOn, CatalogModel
from .discount import Discount
from .transaction import Transaction


class Subscription(CatalogModel):
    add_ons: List[AddOn]
    balance: Decimal
    billing_day_of_month: Decimal
    billing_period_start_date: date
    billing_period_end_date: date
    created_at: datetime
    current_billing_cycle: Decimal
    days_past_due: Decimal
    description: str
    discounts: List[Discount]
    failure_count: Decimal
    first_billing_date: date
    id: str
    merchant_account_id: str
    never_expires: bool
    next_bill_amount: Decimal
    next_billing_date: date
    next_billing_period_amount: Decimal
    number_of_billing_cycles: Decimal
    paid_through_date: date
    payment_method_token: str
    plan_id: str
    price: Decimal
    status: str
    transactions: List[Transaction]
    trial_duration: Decimal
    trial_duration_unit: str
    trial_period: bool
    updated_at: datetime



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#


from .cards import Address
from .common import CatalogModel


class BussinessDetails(CatalogModel):
    address_details: Address
    dba_name: str
    legal_name: str
    tax_id: str


class FundingDetails(CatalogModel):
    account_number_last_4: str
    descriptor: str
    destination: str
    email: str
    mobile_phone: str
    routing_number: str


class IndividualDetails(CatalogModel):
    address_details: Address
    date_of_birth: str
    email: str
    first_name: str
    last_name: str
    phone: str
    ssn_last_4: str


class MerchantAccount(CatalogModel):
    business_details: BussinessDetails
    currency_iso_code: str
    funding_details: FundingDetails
    id: str
    individual_details: IndividualDetails
    status: str



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from datetime import date, datetime
from decimal import Decimal
from typing import List, Optional, Union

from .common import AllOptional, CatalogModel


class Evidence(CatalogModel, metaclass=AllOptional):
    created_at: datetime
    id: str
    sent_to_processor_at: datetime
    url: str
    comment: str


class PaypalMessage(CatalogModel):
    message: str
    send_at: datetime
    sender: str


class Dispute(CatalogModel):
    amount_disputed: Decimal
    amount_won: Decimal
    case_number: str
    chargeback_protection_level: Optional[str]
    created_at: datetime
    currency_iso_code: str
    evidence: Union[Evidence, List[Evidence]]
    graphql_id: str
    id: str
    kind: str
    merchant_account_id: str
    original_dispute_id: Optional[str]
    paypal_messages: List[PaypalMessage]
    processor_comments: Optional[str]
    reason: str
    reason_code: str
    reason_description: Optional[str]
    received_date: date
    reference_number: Optional[str]
    reply_by_date: date
    status: str
    updated_at: datetime
